package ru.mai.dep806.catalog.model;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.Cascade;

@javax.persistence.SequenceGenerator(
    name="SEQ_ID",
    sequenceName="SEQ_ID"
)
@Entity @Table(name="CATALOG_ITEM")
@Inheritance(strategy=InheritanceType.SINGLE_TABLE)
@DiscriminatorColumn(
    name="ITEM_TYPE",
    discriminatorType=DiscriminatorType.STRING
)
public abstract class CatalogItem {

	private Long id;
	
	/**
	 * @clientCardinality 0..*
	 * @link association
	 * @supplierCardinality 1
	 */
	
	private Author author;
	private String name;
	
	/**
	 * @clientCardinality 0..*
	 * @link aggregation
	 * @supplierCardinality 1
	 */
	
	private Catalog catalog;
	
	@Id @GeneratedValue(generator="SEQ_ID")
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}

	@ManyToOne(fetch=FetchType.LAZY)
	@Cascade(org.hibernate.annotations.CascadeType.SAVE_UPDATE)
	@JoinColumn(name="AUTHOR_ID")
	public Author getAuthor() {
		return author;
	}
	public void setAuthor(Author author) {
		this.author = author;
	}
		
	@Column(name="NAME")
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	@ManyToOne(cascade=CascadeType.REFRESH, fetch=FetchType.LAZY)
    @JoinColumn(insertable=false, updatable=false)
    public Catalog getCatalog() {
		return catalog;
	}
	public void setCatalog(Catalog catalog) {
		this.catalog = catalog;
	}
	
	
}
